@@ -1,13 +1,16 @@
 import logging
+import uuid
 from typing import List, Generator
 from operator import itemgetter
 from glob import iglob
 from os import listdir, remove, environ, makedirs
 from os.path import exists as os_exists, dirname, getctime
 from shutil import copyfile
+from datetime import datetime, timedelta, timezone
 
 from retrying import retry
 from s3fs import S3FileSystem as OriginalS3FileSystem
+import boto3
 
 from shub_workflow.deliver import gcstorage
 
@@ -64,14 +67,14 @@ def s3_credentials(key, secret, include_region=False):
 
 def get_file(path, *args, aws_key=None, aws_secret=None, **kwargs):
     if path.startswith(_S3_ATTRIBUTE):
-        fs = S3FileSystem(**s3_credentials(aws_key, aws_secret))
-        fp = fs.open(s3_path(path), *args, **kwargs)
+        fs = S3FileSystem(**s3_credentials(aws_key, aws_secret), **kwargs)
+        fp = fs.open(s3_path(path), *args)
         try:
             fp.name = path
         except Exception:
             pass
     else:
-        fp = open(path, *args, **kwargs)
+        fp = open(path, *args)
 
     return fp
 
@@ -104,9 +107,9 @@ def upload_file(path, dest, aws_key=None, aws_secret=None, **kwargs):
                 break
 
 
-def get_glob(path, aws_key=None, aws_secret=None):
+def get_glob(path, aws_key=None, aws_secret=None, **kwargs):
     if path.startswith(_S3_ATTRIBUTE):
-        fs = S3FileSystem(**s3_credentials(aws_key, aws_secret))
+        fs = S3FileSystem(**s3_credentials(aws_key, aws_secret), **kwargs)
         fp = [_S3_ATTRIBUTE + p for p in fs.glob(s3_path(path))]
     else:
         fp = iglob(path)
@@ -114,9 +117,9 @@ def get_glob(path, aws_key=None, aws_secret=None):
     return fp
 
 
-def cp_file(src_path, dest_path, aws_key=None, aws_secret=None):
+def cp_file(src_path, dest_path, aws_key=None, aws_secret=None, **kwargs):
     if src_path.startswith(_S3_ATTRIBUTE):
-        fs = S3FileSystem(**s3_credentials(aws_key, aws_secret))
+        fs = S3FileSystem(**s3_credentials(aws_key, aws_secret), **kwargs)
         fs.copy(s3_path(src_path), s3_path(dest_path))
     else:
         copyfile(src_path, dest_path)
@@ -142,17 +145,17 @@ def mv_file(src_path, dest_path, aws_key=None, aws_secret=None, **kwargs):
         rm_file(src_path)
 
 
-def rm_file(path, aws_key=None, aws_secret=None):
+def rm_file(path, aws_key=None, aws_secret=None, **kwargs):
     if path.startswith(_S3_ATTRIBUTE):
-        fs = S3FileSystem(**s3_credentials(aws_key, aws_secret))
+        fs = S3FileSystem(**s3_credentials(aws_key, aws_secret), **kwargs)
         fs.rm(s3_path(path))
     else:
         remove(path)
 
 
-def list_folder(path, aws_key=None, aws_secret=None) -> List[str]:
+def list_folder(path, aws_key=None, aws_secret=None, **kwargs) -> List[str]:
     if path.startswith(_S3_ATTRIBUTE):
-        fs = S3FileSystem(**s3_credentials(aws_key, aws_secret))
+        fs = S3FileSystem(**s3_credentials(aws_key, aws_secret), **kwargs)
 
         try:
             listing = [f"s3://{name}" for name in fs.ls(s3_path(path, is_folder=True))]
@@ -170,9 +173,9 @@ def list_folder(path, aws_key=None, aws_secret=None) -> List[str]:
     return listing
 
 
-def list_folder_in_ts_order(input_folder: str) -> Generator[str, None, None]:
+def list_folder_in_ts_order(input_folder: str, aws_key=None, aws_secret=None, **kwargs) -> Generator[str, None, None]:
     results = []
-    for input_file in list_folder(input_folder):
+    for input_file in list_folder(input_folder, aws_key=aws_key, aws_secret=aws_secret, **kwargs):
         if input_folder.startswith(_S3_ATTRIBUTE):
             with get_file(input_file, "rb") as f:
                 results.append((input_file, f.info()["LastModified"]))
@@ -182,31 +185,31 @@ def list_folder_in_ts_order(input_folder: str) -> Generator[str, None, None]:
         yield n
 
 
-def list_folder_files_recursive(folder: str) -> Generator[str, None, None]:
-    for f in list_folder(folder):
+def list_folder_files_recursive(folder: str, aws_key=None, aws_secret=None, **kwargs) -> Generator[str, None, None]:
+    for f in list_folder(folder, aws_key=aws_key, aws_secret=aws_secret, **kwargs):
         if f == folder:
             yield f
         else:
-            yield from list_folder_files_recursive(f)
+            yield from list_folder_files_recursive(f, aws_key=aws_key, aws_secret=aws_secret, **kwargs)
 
 
-def s3_folder_size(path, aws_key=None, aws_secret=None):
+def s3_folder_size(path, aws_key=None, aws_secret=None, **kwargs):
     if path.startswith(_S3_ATTRIBUTE):
-        fs = S3FileSystem(**s3_credentials(aws_key, aws_secret))
+        fs = S3FileSystem(**s3_credentials(aws_key, aws_secret), **kwargs)
         return sum(fs.du(s3_path(path, is_folder=True), deep=True).values())
 
 
-def exists(path, aws_key=None, aws_secret=None):
+def exists(path, aws_key=None, aws_secret=None, **kwargs):
     if path.startswith(_S3_ATTRIBUTE):
-        fs = S3FileSystem(**s3_credentials(aws_key, aws_secret))
+        fs = S3FileSystem(**s3_credentials(aws_key, aws_secret), **kwargs)
         return fs.exists(path)
     return os_exists(path)
 
 
-def empty_folder(path, aws_key=None, aws_secret=None) -> List[str]:
+def empty_folder(path, aws_key=None, aws_secret=None, **kwargs) -> List[str]:
     removed_files = []
     if path.startswith(_S3_ATTRIBUTE):
-        fs = S3FileSystem(**s3_credentials(aws_key, aws_secret))
+        fs = S3FileSystem(**s3_credentials(aws_key, aws_secret), **kwargs)
         for s3file in list_folder(path, aws_key=aws_key, aws_secret=aws_secret):
             try:
                 fs.rm(s3file)
@@ -223,12 +226,48 @@ def empty_folder(path, aws_key=None, aws_secret=None) -> List[str]:
     return removed_files
 
 
-def touch(path, aws_key=None, aws_secret=None):
+def touch(path, aws_key=None, aws_secret=None, **kwargs):
     if not exists(path):
         if not path.startswith(_S3_ATTRIBUTE):
             dname = dirname(path)
             makedirs(dname, exist_ok=True)
-        with get_file(path, "w", aws_key=aws_key, aws_secret=aws_secret) as f:
+        with get_file(path, "w", aws_key=aws_key, aws_secret=aws_secret, **kwargs) as f:
             f.write("")
     else:
         raise ValueError(f"File {path} already exists")
+
+
+class S3SessionFactory:
+
+    """
+    Generate s3 temporal session credentials in cases where that is required
+    """
+
+    def __init__(self, aws_key, aws_secret, aws_role, external_id, expiration_margin_minutes=10):
+        self.aws_role = aws_role
+        self.aws_key = aws_key
+        self.aws_secret = aws_secret
+        self.external_id = external_id
+        self.expiration_margin_minutes = expiration_margin_minutes
+        self.credentials_expiration = None
+        self.sts_client = boto3.client("sts", aws_access_key_id=aws_key, aws_secret_access_key=aws_secret)
+        self.full_credentials = {}
+
+    def get_credentials(self):
+        """
+        Get a new set of credentials if the existing ones are close to expire,
+        update the S3 client.
+        """
+        if self.credentials_expiration is None or datetime.now(timezone.utc) > self.credentials_expiration - timedelta(
+            minutes=self.expiration_margin_minutes
+        ):
+            session_name = str(uuid.uuid4())
+            self.full_credentials = self.sts_client.assume_role(
+                RoleArn=self.aws_role, RoleSessionName=session_name, ExternalId=self.external_id,
+            )["Credentials"]
+            self.credentials_expiration = self.full_credentials["Expiration"]
+        return {
+            "aws_key": self.full_credentials["AccessKeyId"],
+            "aws_secret": self.full_credentials["SecretAccessKey"],
+            "token": self.full_credentials["SessionToken"],
+        }
